package sun.rmi.server;

import java.util.Enumeration;
import java.lang.ref.WeakReference;
import java.util.WeakHashMap;
import java.util.Collections;
import java.util.IdentityHashMap;
import java.rmi.server.LogStream;
import sun.security.action.GetPropertyAction;
import sun.reflect.misc.ReflectUtil;
import java.net.URLConnection;
import java.io.File;
import java.net.SocketPermission;
import java.net.JarURLConnection;
import java.io.FilePermission;
import java.security.ProtectionDomain;
import java.util.PropertyPermission;
import java.security.Policy;
import java.security.CodeSource;
import java.security.cert.Certificate;
import java.security.PermissionCollection;
import java.security.AccessControlContext;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.lang.ref.SoftReference;
import java.util.StringTokenizer;
import java.lang.reflect.Modifier;
import java.lang.reflect.Proxy;
import java.util.Arrays;
import java.security.Permission;
import java.io.IOException;
import java.security.Permissions;
import java.net.URLClassLoader;
import java.net.MalformedURLException;
import java.lang.ref.ReferenceQueue;
import java.util.HashMap;
import java.util.Map;
import java.net.URL;
import sun.rmi.runtime.Log;

public final class LoaderHandler
{
    static final int logLevel;
    static final Log loaderLog;
    private static String codebaseProperty;
    private static URL[] codebaseURLs;
    private static final Map<ClassLoader, Void> codebaseLoaders;
    private static final HashMap<LoaderHandler.LoaderKey, LoaderHandler.LoaderEntry> loaderTable;
    private static final ReferenceQueue<LoaderHandler.Loader> refQueue;
    private static final Map<String, Object[]> pathToURLsCache;
    
    private LoaderHandler() {
    }
    
    private static synchronized URL[] getDefaultCodebaseURLs() throws MalformedURLException {
        if (LoaderHandler.codebaseURLs == null) {
            if (LoaderHandler.codebaseProperty != null) {
                LoaderHandler.codebaseURLs = pathToURLs(LoaderHandler.codebaseProperty);
            }
            else {
                LoaderHandler.codebaseURLs = new URL[0];
            }
        }
        return LoaderHandler.codebaseURLs;
    }
    
    public static Class<?> loadClass(String s, String s2, ClassLoader obj) throws MalformedURLException, ClassNotFoundException {
        if (LoaderHandler.loaderLog.isLoggable(Log.BRIEF)) {
            LoaderHandler.loaderLog.log(Log.BRIEF, "name = \"" + s2 + "\", codebase = \"" + ((s != null) ? s : "") + "\"" + ((obj != null) ? (", defaultLoader = " + obj) : ""));
        }
        URL[] array;
        if (s != null) {
            array = pathToURLs(s);
        }
        else {
            array = getDefaultCodebaseURLs();
        }
        if (obj != null) {
            try {
                Class<Object> loadClassForName = loadClassForName(s2, false, obj);
                if (LoaderHandler.loaderLog.isLoggable(Log.VERBOSE)) {
                    LoaderHandler.loaderLog.log(Log.VERBOSE, "class \"" + s2 + "\" found via defaultLoader, defined by " + loadClassForName.getClassLoader());
                    return loadClassForName;
                }
                return loadClassForName;
            }
            catch (ClassNotFoundException ex) {}
        }
        return loadClass(array, s2);
    }
    
    public static String getClassAnnotation(Class<?> clazz) {
        String name = clazz.getName();
        int length = name.length();
        if (length > 0 && name.charAt(0) == '[') {
            int n;
            for (n = 1; length > n && name.charAt(n) == '['; ++n) {}
            if (length > n && name.charAt(n) != 'L') {
                return null;
            }
        }
        ClassLoader classLoader = clazz.getClassLoader();
        if (classLoader == null || LoaderHandler.codebaseLoaders.containsKey(classLoader)) {
            return LoaderHandler.codebaseProperty;
        }
        String s = null;
        if ((classLoader instanceof LoaderHandler.Loader)) {
            s = ((LoaderHandler.Loader)classLoader).getClassAnnotation();
        }
        else if ((classLoader instanceof URLClassLoader)) {
            try {
                URL[] urLs = ((URLClassLoader)classLoader).getURLs();
                if (urLs != null) {
                    SecurityManager securityManager = System.getSecurityManager();
                    if (securityManager != null) {
                        Permissions permissions = new Permissions();
                        for (int i = 0; i < urLs.length; ++i) {
                            Permission permission = urLs[i].openConnection().getPermission();
                            if (permission != null && !permissions.implies(permission)) {
                                securityManager.checkPermission(permission);
                                permissions.add(permission);
                            }
                        }
                    }
                    s = urlsToPath(urLs);
                }
            }
            catch (SecurityException | IOException ex) {}
        }
        if (s != null) {
            return s;
        }
        return LoaderHandler.codebaseProperty;
    }
    
    public static ClassLoader getClassLoader(String s) throws MalformedURLException {
        ClassLoader rmiContextClassLoader = getRMIContextClassLoader();
        URL[] array;
        if (s != null) {
            array = pathToURLs(s);
        }
        else {
            array = getDefaultCodebaseURLs();
        }
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(new RuntimePermission("getClassLoader"));
            LoaderHandler.Loader lookupLoader = lookupLoader(array, rmiContextClassLoader);
            if (lookupLoader != null) {
                LoaderHandler.Loader.access$000(lookupLoader);
                return lookupLoader;
            }
            return lookupLoader;
        }
        else {
            return rmiContextClassLoader;
        }
    }
    
    public static Object getSecurityContext(ClassLoader classLoader) {
        if ((classLoader instanceof LoaderHandler.Loader)) {
            URL[] urLs = ((LoaderHandler.Loader)classLoader).getURLs();
            if (urLs.length > 0) {
                return urLs[0];
            }
        }
        return null;
    }
    
    public static void registerCodebaseLoader(ClassLoader classLoader) {
        LoaderHandler.codebaseLoaders.put(classLoader, null);
    }
    
    private static Class<?> loadClass(URL[] array, String str) throws ClassNotFoundException {
        ClassLoader rmiContextClassLoader = getRMIContextClassLoader();
        if (LoaderHandler.loaderLog.isLoggable(Log.VERBOSE)) {
            LoaderHandler.loaderLog.log(Log.VERBOSE, "(thread context class loader: " + rmiContextClassLoader + ")");
        }
        if (System.getSecurityManager() == null) {
            try {
                Class<Object> forName = Class.forName(str, false, rmiContextClassLoader);
                if (LoaderHandler.loaderLog.isLoggable(Log.VERBOSE)) {
                    LoaderHandler.loaderLog.log(Log.VERBOSE, "class \"" + str + "\" found via thread context class loader (no security manager: codebase disabled), defined by " + forName.getClassLoader());
                    return forName;
                }
                return forName;
            }
            catch (ClassNotFoundException ex4) {
                if (LoaderHandler.loaderLog.isLoggable(Log.BRIEF)) {
                    LoaderHandler.loaderLog.log(Log.BRIEF, "class \"" + str + "\" not found via thread context class loader (no security manager: codebase disabled)", ex4);
                }
                throw new ClassNotFoundException(ex4.getMessage() + " (no security manager: RMI class loader disabled)", ex4.getException());
            }
        }
        else {
            LoaderHandler.Loader lookupLoader = lookupLoader(array, rmiContextClassLoader);
            try {
                if (lookupLoader != null) {
                    LoaderHandler.Loader.access$000(lookupLoader);
                }
            }
            catch (SecurityException ex) {
                try {
                    Class<Object> loadClassForName = loadClassForName(str, false, rmiContextClassLoader);
                    if (LoaderHandler.loaderLog.isLoggable(Log.VERBOSE)) {
                        LoaderHandler.loaderLog.log(Log.VERBOSE, "class \"" + str + "\" found via thread context class loader (access to codebase denied), defined by " + loadClassForName.getClassLoader());
                        return loadClassForName;
                    }
                    return loadClassForName;
                }
                catch (ClassNotFoundException ex2) {
                    if (LoaderHandler.loaderLog.isLoggable(Log.BRIEF)) {
                        LoaderHandler.loaderLog.log(Log.BRIEF, "class \"" + str + "\" not found via thread context class loader (access to codebase denied)", ex);
                    }
                    throw new ClassNotFoundException("access to class loader denied", ex);
                }
            }
            try {
                Class<Object> loadClassForName2 = loadClassForName(str, false, lookupLoader);
                if (LoaderHandler.loaderLog.isLoggable(Log.VERBOSE)) {
                    LoaderHandler.loaderLog.log(Log.VERBOSE, "class \"" + str + "\" found via codebase, defined by " + loadClassForName2.getClassLoader());
                    return loadClassForName2;
                }
                return loadClassForName2;
            }
            catch (ClassNotFoundException ex3) {
                if (LoaderHandler.loaderLog.isLoggable(Log.BRIEF)) {
                    LoaderHandler.loaderLog.log(Log.BRIEF, "class \"" + str + "\" not found via codebase", ex3);
                }
                throw ex3;
            }
        }
    }
    
    public static Class<?> loadProxyClass(String s, String[] a, ClassLoader obj) throws MalformedURLException, ClassNotFoundException {
        if (LoaderHandler.loaderLog.isLoggable(Log.BRIEF)) {
            LoaderHandler.loaderLog.log(Log.BRIEF, "interfaces = " + Arrays.asList(a) + ", codebase = \"" + ((s != null) ? s : "") + "\"" + ((obj != null) ? (", defaultLoader = " + obj) : ""));
        }
        ClassLoader rmiContextClassLoader = getRMIContextClassLoader();
        if (LoaderHandler.loaderLog.isLoggable(Log.VERBOSE)) {
            LoaderHandler.loaderLog.log(Log.VERBOSE, "(thread context class loader: " + rmiContextClassLoader + ")");
        }
        URL[] array;
        if (s != null) {
            array = pathToURLs(s);
        }
        else {
            array = getDefaultCodebaseURLs();
        }
        if (System.getSecurityManager() == null) {
            try {
                Class<Object> loadProxyClass3 = loadProxyClass(a, obj, rmiContextClassLoader, false);
                if (LoaderHandler.loaderLog.isLoggable(Log.VERBOSE)) {
                    LoaderHandler.loaderLog.log(Log.VERBOSE, "(no security manager: codebase disabled) proxy class defined by " + loadProxyClass3.getClassLoader());
                    return loadProxyClass3;
                }
                return loadProxyClass3;
            }
            catch (ClassNotFoundException ex4) {
                if (LoaderHandler.loaderLog.isLoggable(Log.BRIEF)) {
                    LoaderHandler.loaderLog.log(Log.BRIEF, "(no security manager: codebase disabled) proxy class resolution failed", ex4);
                }
                throw new ClassNotFoundException(ex4.getMessage() + " (no security manager: RMI class loader disabled)", ex4.getException());
            }
        }
        else {
            LoaderHandler.Loader lookupLoader = lookupLoader(array, rmiContextClassLoader);
            try {
                if (lookupLoader != null) {
                    LoaderHandler.Loader.access$000(lookupLoader);
                }
            }
            catch (SecurityException ex) {
                try {
                    Class<Object> loadProxyClass = loadProxyClass(a, obj, rmiContextClassLoader, false);
                    if (LoaderHandler.loaderLog.isLoggable(Log.VERBOSE)) {
                        LoaderHandler.loaderLog.log(Log.VERBOSE, "(access to codebase denied) proxy class defined by " + loadProxyClass.getClassLoader());
                        return loadProxyClass;
                    }
                    return loadProxyClass;
                }
                catch (ClassNotFoundException ex2) {
                    if (LoaderHandler.loaderLog.isLoggable(Log.BRIEF)) {
                        LoaderHandler.loaderLog.log(Log.BRIEF, "(access to codebase denied) proxy class resolution failed", ex);
                    }
                    throw new ClassNotFoundException("access to class loader denied", ex);
                }
            }
            try {
                Class<Object> loadProxyClass2 = loadProxyClass(a, obj, lookupLoader, true);
                if (LoaderHandler.loaderLog.isLoggable(Log.VERBOSE)) {
                    LoaderHandler.loaderLog.log(Log.VERBOSE, "proxy class defined by " + loadProxyClass2.getClassLoader());
                    return loadProxyClass2;
                }
                return loadProxyClass2;
            }
            catch (ClassNotFoundException ex3) {
                if (LoaderHandler.loaderLog.isLoggable(Log.BRIEF)) {
                    LoaderHandler.loaderLog.log(Log.BRIEF, "proxy class resolution failed", ex3);
                }
                throw ex3;
            }
        }
    }
    
    private static Class<?> loadProxyClass(String[] array, ClassLoader classLoader, ClassLoader loader, boolean b) throws ClassNotFoundException {
        Class[] interfaces = new Class[array.length];
        boolean[] array2 = { false };
        Label_0155: {
            if (classLoader != null) {
                ClassLoader loadProxyInterfaces;
                try {
                    loadProxyInterfaces = loadProxyInterfaces(array, classLoader, interfaces, array2);
                    if (LoaderHandler.loaderLog.isLoggable(Log.VERBOSE)) {
                        ClassLoader[] a = new ClassLoader[interfaces.length];
                        for (int i = 0; i < a.length; ++i) {
                            a[i] = interfaces[i].getClassLoader();
                        }
                        LoaderHandler.loaderLog.log(Log.VERBOSE, "proxy interfaces found via defaultLoader, defined by " + Arrays.asList(a));
                    }
                }
                catch (ClassNotFoundException ex) {
                    break Label_0155;
                }
                if (!array2[0]) {
                    if (b) {
                        try {
                            return Proxy.getProxyClass(loader, interfaces);
                        }
                        catch (IllegalArgumentException ex2) {}
                    }
                    loadProxyInterfaces = classLoader;
                }
                return loadProxyClass(loadProxyInterfaces, interfaces);
            }
        }
        array2[0] = false;
        ClassLoader loadProxyInterfaces2 = loadProxyInterfaces(array, loader, interfaces, array2);
        if (LoaderHandler.loaderLog.isLoggable(Log.VERBOSE)) {
            ClassLoader[] a2 = new ClassLoader[interfaces.length];
            for (int j = 0; j < a2.length; ++j) {
                a2[j] = interfaces[j].getClassLoader();
            }
            LoaderHandler.loaderLog.log(Log.VERBOSE, "proxy interfaces found via codebase, defined by " + Arrays.asList(a2));
        }
        if (!array2[0]) {
            loadProxyInterfaces2 = loader;
        }
        return loadProxyClass(loadProxyInterfaces2, interfaces);
    }
    
    private static Class<?> loadProxyClass(ClassLoader loader, Class<?>[] interfaces) throws ClassNotFoundException {
        try {
            return Proxy.getProxyClass(loader, interfaces);
        }
        catch (IllegalArgumentException ex) {
            throw new ClassNotFoundException("error creating dynamic proxy class", ex);
        }
    }
    
    private static ClassLoader loadProxyInterfaces(String[] array, ClassLoader classLoader, Class<?>[] array2, boolean[] array3) throws ClassNotFoundException {
        ClassLoader classLoader2 = null;
        for (int i = 0; i < array.length; ++i) {
            int n = i;
            Class<?> loadClassForName = loadClassForName(array[i], false, classLoader);
            array2[n] = loadClassForName;
            Class<Object> clazz = loadClassForName;
            if (!Modifier.isPublic(clazz.getModifiers())) {
                ClassLoader classLoader3 = clazz.getClassLoader();
                if (LoaderHandler.loaderLog.isLoggable(Log.VERBOSE)) {
                    LoaderHandler.loaderLog.log(Log.VERBOSE, "non-public interface \"" + array[i] + "\" defined by " + classLoader3);
                }
                if (!array3[0]) {
                    classLoader2 = classLoader3;
                    array3[0] = true;
                }
                else if (classLoader3 != classLoader2) {
                    throw new IllegalAccessError("non-public interfaces defined in different class loaders");
                }
            }
        }
        return classLoader2;
    }
    
    private static URL[] pathToURLs(String s) throws MalformedURLException {
        synchronized (LoaderHandler.pathToURLsCache) {
            Object[] array = (Object[])LoaderHandler.pathToURLsCache.get(s);
            if (array != null) {
                return (URL[])(URL[])array[0];
            }
        }
        StringTokenizer stringTokenizer = new StringTokenizer(s);
        URL[] array3 = new URL[stringTokenizer.countTokens()];
        int n = 0;
        while (stringTokenizer.hasMoreTokens()) {
            array3[n] = new URL(stringTokenizer.nextToken());
            ++n;
        }
        synchronized (LoaderHandler.pathToURLsCache) {
            LoaderHandler.pathToURLsCache.put(s, new Object[] { array3, new SoftReference(s) });
        }
        return array3;
    }
    
    private static String urlsToPath(URL[] array) {
        if (array.length == 0) {
            return null;
        }
        if (array.length == 1) {
            return array[0].toExternalForm();
        }
        StringBuffer sb = new StringBuffer(array[0].toExternalForm());
        for (int i = 1; i < array.length; ++i) {
            sb.append(' ');
            sb.append(array[i].toExternalForm());
        }
        return sb.toString();
    }
    
    private static ClassLoader getRMIContextClassLoader() {
        return Thread.currentThread().getContextClassLoader();
    }
    
    private static LoaderHandler.Loader lookupLoader(final URL[] array, final ClassLoader classLoader) {
        LoaderHandler.Loader loader;
        synchronized (LoaderHandler.class) {
            LoaderHandler.LoaderEntry loaderEntry;
            while ((loaderEntry = (LoaderHandler.LoaderEntry)LoaderHandler.refQueue.poll()) != null) {
                if (loaderEntry.removed) {
                    continue;
                }
                LoaderHandler.loaderTable.remove(loaderEntry.key);
            }
            LoaderHandler.LoaderKey key = new LoaderHandler.LoaderKey(array, classLoader);
            LoaderHandler.LoaderEntry loaderEntry2 = (LoaderHandler.LoaderEntry)LoaderHandler.loaderTable.get(key);
            if (loaderEntry2 == null || (loader = (LoaderHandler.Loader)loaderEntry2.get()) == null) {
                if (loaderEntry2 != null) {
                    LoaderHandler.loaderTable.remove(key);
                    loaderEntry2.removed = true;
                }
                loader = (LoaderHandler.Loader)AccessController.doPrivileged(new PrivilegedAction<LoaderHandler.Loader>() {
                    @Override
                    public LoaderHandler.Loader run() {
                        return new LoaderHandler.Loader(array, classLoader);
                    }
                }, getLoaderAccessControlContext(array));
                LoaderHandler.loaderTable.put(key, new LoaderHandler.LoaderEntry(key, loader));
            }
        }
        return loader;
    }
    
    private static AccessControlContext getLoaderAccessControlContext(URL[] array) {
        PermissionCollection permissions = (PermissionCollection)AccessController.doPrivileged(new PrivilegedAction<PermissionCollection>() {
            @Override
            public PermissionCollection run() {
                CodeSource codesource = new CodeSource(null, (Certificate[])null);
                Policy policy = Policy.getPolicy();
                if (policy != null) {
                    return policy.getPermissions(codesource);
                }
                return new Permissions();
            }
        });
        permissions.add(new RuntimePermission("createClassLoader"));
        permissions.add(new PropertyPermission("java.*", "read"));
        addPermissionsForURLs(array, permissions, true);
        return new AccessControlContext(new ProtectionDomain[] { new ProtectionDomain(new CodeSource((array.length > 0) ? array[0] : null, (Certificate[])null), permissions) });
    }
    
    private static void addPermissionsForURLs(URL[] array, PermissionCollection collection, boolean b) {
        for (int i = 0; i < array.length; ++i) {
            URL jarFileURL = array[i];
            try {
                URLConnection urlConnection = jarFileURL.openConnection();
                Permission permission = urlConnection.getPermission();
                if (permission != null) {
                    if ((permission instanceof FilePermission)) {
                        String name = permission.getName();
                        int lastIndex = name.lastIndexOf((int)File.separatorChar);
                        if (lastIndex != -1) {
                            String path = name.substring(0, lastIndex + 1);
                            if (path.endsWith(File.separator)) {
                                path += "-";
                            }
                            FilePermission o2 = new FilePermission(path, "read");
                            if (!collection.implies(o2)) {
                                collection.add(o2);
                            }
                            collection.add(new FilePermission(path, "read"));
                        }
                        else if (!collection.implies(permission)) {
                            collection.add(permission);
                        }
                    }
                    else {
                        if (!collection.implies(permission)) {
                            collection.add(permission);
                        }
                        if (b) {
                            for (, ; urlConnection instanceof JarURLConnection; urlConnection = jarFileURL.openConnection()) {
                                jarFileURL = ((JarURLConnection)urlConnection).getJarFileURL();
                            }
                            String host = jarFileURL.getHost();
                            if (host != null && permission.implies(new SocketPermission(host, "resolve"))) {
                                SocketPermission o = new SocketPermission(host, "connect,accept");
                                if (!collection.implies(o)) {
                                    collection.add(o);
                                }
                            }
                        }
                    }
                }
            }
            catch (IOException ex) {}
        }
    }
    
    private static Class<?> loadClassForName(String s, boolean initialize, ClassLoader loader) throws ClassNotFoundException {
        if (loader == null) {
            ReflectUtil.checkPackageAccess(s);
        }
        return Class.forName(s, initialize, loader);
    }
    
    static void access$300(URL[] array, PermissionCollection collection, boolean b) {
        addPermissionsForURLs(array, collection, b);
    }
    
    static String access$400(URL[] array) {
        return urlsToPath(array);
    }
    
    static {
        logLevel = LogStream.parseLevel((String)AccessController.doPrivileged(new GetPropertyAction("sun.rmi.loader.logLevel")));
        loaderLog = Log.getLog("sun.rmi.loader", "loader", LoaderHandler.logLevel);
        LoaderHandler.codebaseProperty = null;
        String codebaseProperty = (String)AccessController.doPrivileged(new GetPropertyAction("java.rmi.server.codebase"));
        if (codebaseProperty != null && codebaseProperty.trim().length() > 0) {
            LoaderHandler.codebaseProperty = codebaseProperty;
        }
        LoaderHandler.codebaseURLs = null;
        codebaseLoaders = Collections.synchronizedMap(new IdentityHashMap(5));
        for (ClassLoader classLoader = ClassLoader.getSystemClassLoader(); classLoader != null; classLoader = classLoader.getParent()) {
            LoaderHandler.codebaseLoaders.put(classLoader, null);
        }
        loaderTable = new HashMap(5);
        refQueue = new ReferenceQueue();
        pathToURLsCache = new WeakHashMap(5);
    }
    
    private static class LoaderKey
    {
        private URL[] urls;
        private ClassLoader parent;
        private int hashValue;
        
        public LoaderKey(URL[] urls, ClassLoader parent) {
            this.urls = urls;
            this.parent = parent;
            if (parent != null) {
                this.hashValue = parent.hashCode();
            }
            for (int i = 0; i < urls.length; ++i) {
                this.hashValue ^= urls[i].hashCode();
            }
        }
        
        @Override
        public int hashCode() {
            return this.hashValue;
        }
        
        @Override
        public boolean equals(Object o) {
            if ((o instanceof LoaderHandler.LoaderKey)) {
                LoaderHandler.LoaderKey loaderKey = (LoaderHandler.LoaderKey)o;
                if (this.parent != loaderKey.parent) {
                    return false;
                }
                if (this.urls == loaderKey.urls) {
                    return true;
                }
                if (this.urls.length != loaderKey.urls.length) {
                    return false;
                }
                int i = 0;
                while (i < this.urls.length) {
                    if (!this.urls[i].equals(loaderKey.urls[i])) {
                        return false;
                    }
                    ++i;
                }
                return true;
            }
            else {
                return false;
            }
        }
    }
    
    private static class LoaderEntry extends WeakReference<LoaderHandler.Loader>
    {
        public LoaderHandler.LoaderKey key;
        public boolean removed;
        
        public LoaderEntry(LoaderHandler.LoaderKey key, LoaderHandler.Loader referent) {
            super(referent, LoaderHandler.refQueue);
            this.removed = false;
            this.key = key;
        }
    }
    
    private static class Loader extends URLClassLoader
    {
        private ClassLoader parent;
        private String annotation;
        private Permissions permissions;
        
        private Loader(URL[] urls, ClassLoader classLoader) {
            super(urls, classLoader);
            this.parent = classLoader;
            LoaderHandler.access$300(urls, this.permissions = new Permissions(), false);
            this.annotation = LoaderHandler.access$400(urls);
        }
        
        public String getClassAnnotation() {
            return this.annotation;
        }
        
        private void checkPermissions() {
            SecurityManager securityManager = System.getSecurityManager();
            if (securityManager != null) {
                Enumeration<Permission> elements = this.permissions.elements();
                while (elements.hasMoreElements()) {
                    securityManager.checkPermission((Permission)elements.nextElement());
                }
            }
        }
        
        @Override
        protected PermissionCollection getPermissions(CodeSource codesource) {
            return super.getPermissions(codesource);
        }
        
        @Override
        public String toString() {
            return super.toString() + "[\"" + this.annotation + "\"]";
        }
        
        @Override
        protected Class<?> loadClass(String s, boolean resolve) throws ClassNotFoundException {
            if (this.parent == null) {
                ReflectUtil.checkPackageAccess(s);
            }
            return super.loadClass(s, resolve);
        }
        
        static void access$000(LoaderHandler.Loader loader) {
            loader.checkPermissions();
        }
    }
}
